% MISO_demo.m, Sindre Johan Heggheim, sindrjh@stud.ntnu.no 10.31.2017
clear all
close all
clc

%% Load response models and initialize system parameters:
Ts = 0.1;
com = 'COM4';   % com port
delete(instrfind({'Port'},{com}));
baud = 250000;  % baud rate 
Ts  = 0.1;      % Sampling time period
fTt = 0.05;     % Filter time constant for temperature (0.05s - 10s)
fTl = 0.05;     % Filter time constant for light intensity (0.05s - 10s)
fTf = 0.1;      % Filter time constant for light intensity (0.1s - 10s)
t_sim = 2;      % Simulation time
load('WS_tfs')  

%% Using Bulb and Fan as temperature controllers. 
% System transfer functions:
sys_tf11 = tf_BT;
sys_tf21 = tf_FT;
sys_num11 = cell2mat(sys_tf11.num);        
sys_den11 = cell2mat(sys_tf11.den);
sys_num21 = cell2mat(sys_tf21.num);        
sys_den21 = cell2mat(sys_tf21.den);

% calculate tight PI control parameters
[Kc11,tauI11] = SIMCPI(sys_tf11,0.1);
[Kc21,tauI21] = SIMCPI(sys_tf21,0.1);

%% measure standby temperature
t_step = 20; % no step since t_step > t_sim
t_sim  = 2;
stepBulb = 0;
stepFan = 0;
stepLed = 0;
bo_c21 = 0;
open('runSystem');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS
sb_T = Temperature(1,1); %standby temperature
close_system('runSystem');

%% Test MISO controller
bo_c21 = +0.15; % Back-off on Fan

% Response and rejection:
y0 = sb_T;
y_s0 = 0;
stepTime1 = 30; %no step here
stepTime2 = 60;
step_ys1 = 5;
step_ys2 = -5;
stepTime_d1 = 90;
stepTime_d2 = 120;
step_d1 = -2.5;
step_d2 = +2.5;
t_sim = 150;

% Choose IO
turnOn_u1 = 1; % Bulb in
turnOn_u2 = 1; % Fan in
turnOn_u3 = 0;
turnOn_y1 = 1; % Temp out
turnOn_y2 = 0;
fixed_u1 = 0;
fixed_u2 = 0;
fixed_u3 = 0;

open('runMISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'MISO_presentation.png', 'png');
hgexport(fig,'MISO_presentation.eps')