% Initializes system and performs step respononses and save measured data.
% The three datasets is saved as the following .mat files:
% WS_Bulb.mat, WS_Fan.mat and WS_Led.mat

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, November 20th, 2017

% The script uses runSystem.slx to run the MIMO system that is connected
% with the port spesified as teh 'com' variable. 
% tom1ar8_lib.mdl, tom_lib.mdl and msfun_realtime_pacer.m is also needed
% to run the system.

%% Initialization of system

clc
clear
com = 'COM4';   % com port
delete(instrfind({'Port'},{com}));
baud = 250000;  % baud rate 
Ts = 0.1;       % Sampling time period
fTt = 0.05;     % Filter time constant for temperature (0.05s - 10s)
fTl = 0.05;     % Filter time constant for light intensity (0.05s - 10s)
fTf = 0.1;      % Filter time constant for light intensity (0.1s - 10s)

t_sim = 1400;   % Simulation time
t_step = 100;   % Step time in simulation
stepSize = 1;   % step change in input in simulation

%% Response from Bulb
stepBulb = stepSize;
stepFan = 0;
stepLed = 0;

open('runSystem'); 
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(3) %Waiting for simulink to save to WS
y = [Temperature,LightIntensity];           % Output, y
u = bulbInput;                              % Input, u
dataBulb = iddata(y,u,Ts);
dataBulb.InputName  = {'Bulb'};
dataBulb.OutputName = {'Temperature';'Light'};
timeBulb = time;
save('WS_Bulb.mat','dataBulb','timeBulb');

%% Response from Fan
t_sim = 700;    % Simulation time
t_step = 10;    % Step time
stepBulb = 0;
stepFan = stepSize;
stepLed = 0;

open('runSystem'); 
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(3) %Waiting for simulink to save to WS
y = [Temperature,LightIntensity];           % Output, y
u = fanInput;                               % Input, u
dataFan = iddata(y,u,Ts);
dataFan.InputName  = {'Fan'};
dataFan.OutputName = {'Temperature';'Light'};
timeFan = time;
save('WS_Fan.mat','dataFan','timeFan');

%% Response from LED
t_sim = 50;    % Simulation time
t_step = 10;    % Step time
stepBulb = 0;
stepFan = 0;
stepLed = stepSize;

open('runSystem'); 
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(3) %Waiting for simulink to save to WS
y = [Temperature,LightIntensity];           % Output, y
u = ledInput;                               % Input, u
dataLed = iddata(y,u,Ts);
dataLed.InputName  = {'Led'};
dataLed.OutputName = {'Temperature';'Light'};
timeLed = time;
save('WS_Led.mat','dataLed','timeLed');