% Implementation of MPC control of MIMO plant.
% The nominal operation point was choosen and the identified state space
% model saved in 'WS_MIMO_models' was used to make the MPC.
%
% See also makeMPCobj

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, December 17th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

%% Initialization

clear
clc
run('init')
load('WS_MIMO_models');

% Find steady state at u = [0.5,0.2,0.5]'
% run('steadyState')
stst.y = [31.51,26.50]';
stst.u = [0.5,0.2,0.5]';

% Tune MPC object
Tsc = 0.1; 
ph = 10;        % Prediction horizon in seconds
ch = 1;         % Control horizon in seconds
MPCobj = makeMPCobj(MIMO_tf,stst,Tsc,ph,ch);
MPCobj.Model.Nominal.Y = stst.y;
MPCobj.Model.Nominal.U = stst.u;

%% Test steady state control with MPC
t_sim = 100;
open('LightPlant_mpcModel')
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMIMOcontrol(y,u,ys,us,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'MIMOtestMPC.png', 'png');
% hgexport(fig,'MIMOtestMPC.eps')

%% Test changes setpoints:
t_sim = 100;

% Temperature setpoints:
stepTime_ys1_1 = 30;
step_ys1_1 = +1;
stepTime_ys1_2 = 70;
step_ys1_2 = -1;

% Light Intensity setpoints:
stepTime_ys2_1 = 20;
step_ys2_1 = 5;
stepTime_ys2_2 =  50;
step_ys2_2 = -5;

open('LightPlant_mpcModel')
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result
fig = plotMIMOcontrol(y,u,ys,us,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'MIMOtestMPC2.png', 'png');
% hgexport(fig,'MIMOtestMPC2.eps')