% Tuning and Implementation of MIMO control.
% The MIMO consist of SISO and MISO controllers and switches the focus of
% the Bulb between the two outputs. The estimated temperature from Bulb to
% Light Intensity is used to choose focus.
% The controllers are tuned with SIMC PI
%
% See also SIMCPI

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, December 17th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

clear
close all
clc
run('init.m') % init files for TOM1A system and set Simulink variables to 0
load('WS_tfs') % Load response models and initialize system parameters

%% Tuning of Temperature controllers
% System transfer functions:
sys_tf11 = tf_BT;
sys_tf21 = tf_FT;
sys_num11 = cell2mat(sys_tf11.num);        
sys_den11 = cell2mat(sys_tf11.den);
sys_num21 = cell2mat(sys_tf21.num);        
sys_den21 = cell2mat(sys_tf21.den);

% % calculate PI control parameters    
[Kc11,tauI11] = SIMCPI(sys_tf11,5);         % B-T
[Kc21,tauI21] = SIMCPI(sys_tf21,2);         % F-T
% [altKc11,alt_tauI11] = SIMCPI(sys_tf11,5);
% [altKc21,alt_tauI21] = SIMCPI(sys_tf21,2);

%% Tuning of Light Intensity controllers
sys_tf12 = tf_BLI;
sys_tf32 = tf_LLI;
sys_num11 = cell2mat(sys_tf12.num);        
sys_den11 = cell2mat(sys_tf12.den);
sys_num21 = cell2mat(sys_tf32.num);        
sys_den21 = cell2mat(sys_tf32.den);

% calculate PI control parameters
[Kc12,tauI12] = SIMCPI(sys_tf12,1);
[Kc32,tauI32] = SIMCPI(sys_tf32,0.5);

%% Operation at:
% run('steadyState') % runs system at stst.u to find stst.y
stst.u = [0.5,0.2,0.5]';
stst.y = [31.51,26.50]';

%% Temperature from B-LI control
% calculate steady state gain to estimate temperature from B-LI control
tf_LIT = tf_BT/tf_BLI;
T_est.num = cell2mat(tf_LIT.num);
T_est.den = cell2mat(tf_LIT.den);
stst_BTgain = T_est.num(2)/T_est.den(2);

%% Test controller on real system: Temperature controlled system
bo_c21 = 0.1;
y0 = 27;                % shift temperature
%Shifted steady state setpoints:
y1_s0 = stst.y(1)-y0;   %4.5
y2_s0 = stst.y(2);      %26.5

step_ys1_1 = -2;
stepTime_ys1_1 = 60;
step_ys1_2 = -0;
stepTime_ys1_2 = 0;

step_ys2_1 = -20;
stepTime_ys2_1 = 30;
step_ys2_2 = +10;
stepTime_ys2_2 = 90;

t_sim = 150;
open('runMIMOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMIMOcontrol(y,u,ys,0,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'MIMOtestTemp.png', 'png');
hgexport(fig,'MIMOtestTemp.eps')