function MPCobj = makeMPCobj(MIMO_tf,stst,Tsc,ph,ch)
% Building and tuning of MPC object used to control MIMO system.
% MIMO_tf:  2x3 transfer functions for MIMO system.
% stst:     Structure of steady state conditions with the 3 inputs '.u'
% Tsc:      Timesteps in MPC
% ph:       Prediction horizon in time units
% ch:       Control horizon in time units

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, November 21th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

plant = MIMO_tf;
plant = setmpcsignals(plant,'MV',[1,2,3]); % define all input as MV

% Define MPC object:
nu = 3;
ny = 2;

p = ph/Tsc;                     % number of steps in prediction horizon
m = ch/Tsc;                     % number of steps in control horizon 
MPCobj = mpc(plant,Tsc,p,m);

% Span of I/O variables:
Uspan = ones(nu,1);
[num,den] = tfdata(MIMO_tf);
numcell1 = num(1,1);
numcell2 = num(2,1);
dencell1 = den(1,1);
dencell2 = den(2,1);
num1 = cell2mat(numcell1);
den1 = cell2mat(dencell1);
num2 = cell2mat(numcell2);
den2 = cell2mat(dencell2);
Yspan = [num1(2)/den1(2), num2(2)/den2(2)]';

% Constraints on inputs:
consU = [0,1;0,1;0,1];

% Scaling factor and constraits on u:
for i = 1:nu
    MPCobj.MV(i).ScaleFactor = Uspan(i);
    MPCobj.MV(i).Min = consU(i,1);
    MPCobj.MV(i).Max = consU(i,2);
end%for

% Scaling factor for y:
for i = 1:ny
    MPCobj.OV(i).ScaleFactor = Yspan(i);
end%for

% MV targets:
MPCobj.MV(1).Target = stst.u(1);
MPCobj.MV(2).Target = stst.u(2);
MPCobj.MV(3).Target = stst.u(3);

% Weights
MPCobj.Weights.MV = [1,0,0];
MPCobj.Weights.MVRate = [1,0.1,0.1];

MPCobj.Weights.OV = [1000,1];

review(MPCobj)
end%fun